@@ -1,6 +1,6 @@
 from statefun_tasks.serialisation import DefaultSerialiser
 from statefun_tasks.type_helpers import flink_value_type_for
-from statefun_tasks.messages_pb2 import TaskState, PipelineState, TaskRequest
+from statefun_tasks.messages_pb2 import TaskState, TaskRequest
 from statefun_tasks.protobuf import pack_any
 
 from statefun import kafka_egress_message, message_builder, Context, SdkAddress
@@ -22,7 +22,7 @@ class TaskContext(object):
 
         self.storage = context.storage
         self.task_state = self.storage.task_state or TaskState()
-        self.pipeline_state = self.storage.pipeline_state or PipelineState()
+        self.pipeline_state = self.storage.pipeline_state
         self._task_meta = {}
         self._task_name = None
 
@@ -169,7 +169,9 @@ class TaskContext(object):
 
     def __exit__(self, exc_type, exc_value, traceback):
         self._context.storage.task_state = self.task_state
-        self._context.storage.pipeline_state = self.pipeline_state
+
+        if self.pipeline_state is not None:
+            self._context.storage.pipeline_state = self.pipeline_state
 
     def __str__(self):
         return f'task_name: {self.task_name}, task_id: {self.get_task_id()}, caller: {self.get_caller_id()}'
