@@ -1,5 +1,5 @@
 from statefun_tasks.types import Task, Group, RetryPolicy
-from statefun_tasks.utils import _gen_id, _is_tuple, _task_type_for
+from statefun_tasks.utils import _gen_id, _is_tuple
 from statefun_tasks.pipeline import _Pipeline
 from statefun_tasks.messages_pb2 import TaskRequest, Pipeline
 
@@ -57,14 +57,6 @@ class PipelineBuilder(object):
     def _add_to_group(self, group: Group):
         group.add_to_group(self._pipeline)
 
-    @staticmethod
-    def _unpack_single_tuple_args(args):
-        # send a single argument by itself instead of wrapped inside a tuple
-        if _is_tuple(args) and len(args) == 1:
-            args = args[0]
-
-        return args
-
     def send(self, fun, *args, **kwargs) -> 'PipelineBuilder':
         """
         Adds a task entry for the given Flink Task and arguments
@@ -74,19 +66,22 @@ class PipelineBuilder(object):
         :param kwargs: the task kwargs
         :return: the builder
         """
-        args = self._unpack_single_tuple_args(args)
-        task_type, parameters = self._task_type_and_parameters_for(fun)
-        self._pipeline.append(Task.from_fields(_gen_id(), task_type, args, kwargs, **parameters))
+        try:
+            task = fun.to_task(args, kwargs)
+            self._pipeline.append(task)
+        except AttributeError:
+            raise AttributeError(f'Function {fun.__module__}.{fun.__name__} should be decorated with tasks.bind')
         return self
 
     def set(self, retry_policy: RetryPolicy = None, namespace: str = None,
-            worker_name: str = None) -> 'PipelineBuilder':
+            worker_name: str = None, is_fruitful=None) -> 'PipelineBuilder':
         """
         Sets task properties on the last entry added to the builder
 
         :param option retry_policy: the task retry policy to use
         :param option namespace: the task namespace
         :param option worker_name: the task worker_name
+        :param option is_fruitful: set to false to drop the results of tasks
         :return: the builder
         """
 
@@ -98,6 +93,8 @@ class PipelineBuilder(object):
                 entry.namespace = namespace
             if worker_name is not None:
                 entry.worker_name = worker_name
+            if is_fruitful is not None:
+                entry.is_fruitful = is_fruitful
 
         return self
 
@@ -113,9 +110,11 @@ class PipelineBuilder(object):
         if isinstance(continuation, PipelineBuilder):
             continuation.append_to(self)
         else:
-            args = self._unpack_single_tuple_args(args)
-            task_type, parameters = self._task_type_and_parameters_for(continuation)
-            self._pipeline.append(Task.from_fields(_gen_id(), task_type, args, kwargs, **parameters))
+            try:
+                task = continuation.to_task(args, kwargs)
+                self._pipeline.append(task)
+            except AttributeError:
+                raise AttributeError(f'Function {continuation.__module__}.{continuation.__name__} should be decorated with tasks.bind')
         return self
 
     def get_destination(self):
@@ -155,11 +154,11 @@ class PipelineBuilder(object):
         :param kwargs: the task kwargs
         :return: the builder
         """
-        args = self._unpack_single_tuple_args(args)
-        task_type, parameters = self._task_type_and_parameters_for(finally_action)
-        combined_parameters = {**parameters, 'is_fruitful': False}
-        task_entry = Task.from_fields(_gen_id(), task_type, args, kwargs, is_finally=True, **combined_parameters)
-        self._pipeline.append(task_entry)
+        try:
+            task = finally_action.to_task(args, kwargs, is_finally=True)
+            self._pipeline.append(task)
+        except AttributeError:
+            raise AttributeError(f'Function {finally_action.__module__}.{finally_action.__name__} should be decorated with tasks.bind')
         return self
 
     def to_pipeline(self, serialiser=None, is_fruitful=True):
@@ -245,17 +244,6 @@ class PipelineBuilder(object):
         """
         return not any(self.get_tasks())
 
-    @staticmethod
-    def _task_type_and_parameters_for(fun):
-        try:
-            parameters = fun.defaults()
-            module_name = parameters.get('module_name', None)
-            task_type = _task_type_for(fun, module_name)
-
-            return task_type, parameters
-        except AttributeError:
-            raise AttributeError(f'Function {fun.__module__}.{fun.__name__} should be decorated with tasks.bind')
-
     def __iter__(self):
         return self._pipeline.__iter__()
 
